#include "Arduino.h"
#include "EP11Shield.h"

#ifndef EP11SHIELD_EXTERNAL_ISR
#include <avr/interrupt.h>

ISR(TIMER1_CAPT_vect) {
	// call ISR
	EP11Shield::captureISR();
}
#endif

volatile uint32_t EP11Shield::m_freqcnt = 0;

void EP11Shield::begin()
{
	// set DAC pins as output (with LOW value)
	EP11SHIELD_DAC_DDR |= EP11SHIELD_DAC_MASK << EP11SHIELD_DAC_SHIFT;
	EP11SHIELD_DAC_PORT &= ~(EP11SHIELD_DAC_MASK << EP11SHIELD_DAC_SHIFT);

	// increase PWM frequency of pin 10 to maximum
	TCCR1B = (TCCR1B & ~((1 << CS11) | (1 << CS12))) | (1 << CS10);

	// set PWM to output
	pinMode(EP11SHIELD_PWM_PIN, OUTPUT);

	// set comparator and capture input
	pinMode(EP11SHIELD_ICP_PIN, INPUT);
	pinMode(EP11SHIELD_COMP_PIN, INPUT);
}

void EP11Shield::end() {
	// set DAC pins as input
	EP11SHIELD_DAC_DDR &= ~(EP11SHIELD_DAC_MASK << EP11SHIELD_DAC_SHIFT);
	EP11SHIELD_DAC_PORT &= ~(EP11SHIELD_DAC_MASK << EP11SHIELD_DAC_SHIFT);	

	// reset PWM frequency to defaults
	TCCR1B = (TCCR1B & ~(1 << CS12)) | (1 << CS10) | (1 << CS11);

	// reset PWM pin to input
	pinMode(EP11SHIELD_PWM_PIN, INPUT);
}

void EP11Shield::captureISR() {
	m_freqcnt++;
}

void EP11Shield::DACwrite(uint8_t value) {
	// mask input value
	value = value & EP11SHIELD_DAC_MASK;

	// write PWM value
	analogWrite(EP11SHIELD_PWM_PIN, (256/(EP11SHIELD_DAC_MASK+1)) * value);

	// write DAC network
	EP11SHIELD_DAC_PORT = (EP11SHIELD_DAC_PORT & ~(EP11SHIELD_DAC_MASK << EP11SHIELD_DAC_SHIFT)) | (value << EP11SHIELD_DAC_SHIFT);
}

int EP11Shield::COMPread() {
	return !digitalRead(EP11SHIELD_COMP_PIN);
}

uint32_t EP11Shield::FREQread(unsigned long gatetime) {
	m_freqcnt = 0;
	TCCR1B |= (1 << ICES1);
	TIMSK1 |= (1 << ICIE1);
	delay(gatetime);
	TIMSK1 &= ~(1 << ICIE1);
	TCCR1B &= ~(1 << ICES1);
	return m_freqcnt;
}

static uint32_t EP11Shield::PULSEread() {
	return pulseIn(EP11SHIELD_ICP_PIN, HIGH);
}
